/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * FilterPanel.java
 *
 * Created on Jan 29, 2010, 9:42:20 AM
 */
package filter;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;

/**
 *
 * @author Mike Markowski ab3ap
 */
public class FilterPanel extends javax.swing.JPanel {

    double bfo = 4912.76; // MHz, BFO as measure during assembly.
    boolean crystalMode;
    boolean drawLeftDot, drawRightDot, drawNrDot;
    boolean notchOn;
    char editMode;
    double thetaCrystal, thetaDsp, widestBw;
    final static int CW = 0;
    final static int CW_REV = 1;
    final static int LSB = 2;
    final static int USB = 3;
    final static int RTTY = 4;
    final static int RTTY_REV = 5;
    int crystalSkirt, dspSkirt, xf, yf, widthf, heightf;
    int chosenCrystal[], chosenDsp, chosenMode;
    int clickX, clickY;
    int nrX, nrY, nrStep;
    Color nrGray = new Color(230, 230, 230);
    Color lightGray = new Color(240, 240, 240);
    Color darkGreen = Color.green.darker();
    FilterModel crystalFilter[][];
    FilterModel dspFilter[][];
    Main main;
    SlowShape crystalTrapezoids[][];
    SlowShape dspTrapezoids[][];

    public FilterPanel() {
        this(6, 4, 3, 4);
    }

    /**
     * Creates new form FilterPanel
     * @param numCrystalFilters
     * @param numDspFilters
     */
    public FilterPanel(int numCrystalModes, int numCrystalFilters,
            int numDspModes, int numDspFilters) {
        crystalMode = true;
        chosenCrystal = new int[numCrystalModes];
        chosenDsp = -1;
        chosenMode = CW;
        drawLeftDot = drawRightDot = false;
        notchOn = false;
        editMode = '-'; // Not in edit mode.
        thetaCrystal = Math.toRadians(75); // Angle at crystal trapezoid base.
        thetaDsp = Math.toRadians(75); // Angle at DSP trapezoid base.
        crystalTrapezoids = new SlowShape[numCrystalModes][numCrystalFilters];
        dspTrapezoids = new SlowShape[numDspModes][numDspFilters];
        initComponents();
    }

    public Main getMain() {
        return main;
    }

    public void setMain(Main main) {
        this.main = main;
    }

    public int getChosenCrystal(int mode) {
        return chosenCrystal[mode];
    }

    public int getChosenDsp() {
        return chosenDsp;
    }

    public void setChosenDsp(int chosenDsp) {
        this.chosenDsp = chosenDsp;
    }

    public void setModeCw() {
        chosenMode = CW;
    }

    public void setModeCwRev() {
        chosenMode = CW_REV;
    }

    public void setModeLsb() {
        chosenMode = LSB;
    }

    public void setModeUsb() {
        chosenMode = USB;
    }

    public void setModeRtty() {
        chosenMode = RTTY;
    }

    public void setModeRttyRev() {
        chosenMode = RTTY_REV;
    }

    public int getChosenMode() {
        return chosenMode;
    }

    public boolean isCrystalMode() {
        return crystalMode;
    }

    public void setCrystalMode(boolean crystalMode) {
        this.crystalMode = crystalMode;
        repaint();
    }

    public boolean isNotchOn() {
        return notchOn;
    }

    public void setNotchOn(boolean notchOn) {
        this.notchOn = notchOn;
    }

    public FilterModel getActiveFilter() {
        if (isCrystalMode()) {
            return crystalFilter[chosenMode][chosenCrystal[chosenMode]];
        } else {
            return dspFilter[chosenMode / 2][chosenDsp];
        }
    }

    public void setCrystalFilters(FilterModel[][] filters) {
        crystalFilter = filters;
        updateComponentSizes();
    }

    public void setDspFilters(FilterModel[][] filters) {
        dspFilter = filters;
        updateComponentSizes();
    }

    @Override
    public void paintComponent(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        if (crystalFilter == null) {
            return;
        }
        if (crystalMode) {
            g.setColor(Color.WHITE);
            g.fillRect(0, 0, getWidth(), getHeight());
            drawCrystalOutlines(g2);
            if (chosenCrystal[chosenMode] != -1) {
                drawCrystalFilter(g2);
            }
            if (main != null) {
                main.updateFilter();
            }
        } else {
            g.setColor(new Color(244, 255, 244));
            g.fillRect(0, 0, getWidth(), getHeight());
            if (chosenCrystal[chosenMode] != -1) {
                drawCrystalMask(g2);
            }
            if (chosenDsp != -1) {
                drawDspFilter(g2);
                drawNrLines(g2);
            }
        }
        if (chosenCrystal[chosenMode] != -1 && chosenDsp == -1) {
            // Label crystal filter.
            labelCrystalFilter(g);
        }
        if (chosenDsp != -1) {
            // Label DSP filter.
            labelDspFilter(g);
            // Draw triangle representing autonotch filter.
            if (chosenMode == LSB || chosenMode == USB) {
                drawDspNotch(g2);
            }
        }
    }

    private void drawNrLines(Graphics2D g2) {
        if (dspTrapezoids == null) {
            return;
        }
        if (chosenMode == RTTY || chosenMode == RTTY_REV) {
            return;
        }
        SlowShape s = dspTrapezoids[chosenMode / 2][chosenDsp];
        Rectangle2D r = s.getP().getBounds2D();
        int xRight = (int) (r.getX() + r.getWidth() + 10);
        g2.setColor(nrGray);
        g2.setStroke(new BasicStroke(1));
        nrDotCoords();
        g2.drawLine(nrX, nrY, xRight, nrY);
        String label = "NR " + dspFilter[chosenMode / 2][chosenDsp].
                getNoiseReductionLevel();
        g2.drawString(label, xRight + 5, nrY + 3);
    }

    private void nrDotCoords() {
        int nrLvl =
                dspFilter[chosenMode / 2][chosenDsp].getNoiseReductionLevel();
        nrStep = heightf / 6;
        nrX = xf - 15;
        nrY = yf + heightf - 3 - nrLvl * nrStep;
    }

    private void drawCrystalOutlines(Graphics2D g2) {
        if (crystalTrapezoids == null) {
            return;
        }
        g2.setColor(lightGray);
        for (int i = 0; i < crystalTrapezoids[chosenMode].length; i++) {
            if (crystalTrapezoids[i] == null) {
                return;
            }
            crystalTrapezoids[chosenMode][i].draw(g2);
        }
    }

    private void drawCrystalFilter(Graphics2D g2) {
        if (crystalTrapezoids == null) {
            return;
        }
        SlowShape s = crystalTrapezoids[chosenMode][chosenCrystal[chosenMode]];
        g2.setColor(Color.green);
        s.setAlpha(0.2f);
        s.setFill(true);
        s.draw(g2);
        // Draw darker outline.
        g2.setStroke(new BasicStroke(3));
        s.setAlpha(1);
        s.setFill(false);
        g2.setColor(darkGreen);
        s.draw(g2);
    }

    private void drawCrystalMask(Graphics2D g2) {
        if (crystalTrapezoids == null) {
            return;
        }
        g2.setColor(Color.white);
        SlowShape s = crystalTrapezoids[chosenMode][chosenCrystal[chosenMode]];
        s.setFill(true);
        s.draw(g2);
        s.setFill(false);
        // Draw darker outline.
        g2.setStroke(new BasicStroke(3));
        s.setAlpha(1);
        s.setFill(false);
        g2.setColor(darkGreen);
        s.draw(g2);
    }

    private void drawDspFilter(Graphics2D g2) {
        if (dspTrapezoids == null) {
            return;
        }
        SlowShape s = dspTrapezoids[chosenMode / 2][chosenDsp];
        g2.setColor(Color.orange);
        s.setAlpha(0.2f);
        s.setFill(true);
        s.draw(g2);
        // Draw darker outline.
        g2.setStroke(new BasicStroke(3));
        s.setAlpha(1);
        s.setFill(false);
        g2.setColor(Color.orange);
        s.draw(g2);
        // Draw dots that user can pull to adjust skirt width.
        if (drawLeftDot) {
            Rectangle2D r = s.getP().getBounds2D();
            int radius = 8;
            int halfR = radius / 2;
            g2.fillOval((int) r.getX() - halfR,
                    (int) (r.getY() + r.getHeight() - halfR), 8, 8);
        } else if (drawRightDot) {
            Rectangle2D r = s.getP().getBounds2D();
            int radius = 8;
            int halfR = radius / 2;
            g2.fillOval((int) (r.getX() + r.getWidth() - halfR),
                    (int) (r.getY() + r.getHeight() - halfR), 8, 8);
        } else if (drawNrDot) {
            g2.setColor(nrGray);
            g2.fillOval(nrX - 4, nrY - 4, 8, 8);
        }
    }

    private void labelCrystalFilter(Graphics g) {
        if (crystalTrapezoids == null) {
            return;
        }
        Rectangle2D r =
                crystalTrapezoids[chosenMode][chosenCrystal[chosenMode]].getP().
                getBounds2D();
        g.setColor(darkGreen);
        int hz = (int) (crystalFilter[chosenMode][chosenCrystal[chosenMode]].
                getBandwidth());
        String label = "XFIL: " + hz + " Hz";
        Rectangle2D bb = calculateStringBox(g, label);
        int lx = (int) (r.getX() + r.getWidth() / 2 - bb.getWidth() / 2);
        int ly = (int) bb.getHeight();
        g.drawString(label, lx, yf + heightf + ly);
        g.drawString("500 Hz", xf + crystalSkirt - (int) (bb.getWidth() / 2),
                yf - ly / 2);
        label = (500 + hz) + " Hz";
        g.drawString(label, xf + (int) r.getWidth() - crystalSkirt,
                yf - ly / 2);
    }

    private void labelDspFilter(Graphics g) {
        if (dspTrapezoids == null) {
            return;
        }
        g.setColor(Color.orange);
        int hz = (int) (dspFilter[chosenMode / 2][chosenDsp].getBandwidth());
        String label = "AFIL: " + hz + " Hz";
        Rectangle2D r = dspTrapezoids[chosenMode / 2][chosenDsp].getP().
                getBounds2D();
        Rectangle2D bb = calculateStringBox(g, label);
        int lx = (int) (r.getX() + r.getWidth() / 2 - bb.getWidth() / 2);
        int ly = (int) bb.getHeight();
        g.drawString(label, lx, yf + heightf + ly);
        if (chosenMode == LSB || chosenMode == USB) {
            // Label left and right filter edges for SSB.
            label =
                    (int) dspFilter[chosenMode / 2][chosenDsp].getLowFreqCutoff() +
                    " Hz";
            bb = calculateStringBox(g, label);
            g.drawString(label,
                    (int) (r.getX() + dspSkirt - bb.getWidth()),
                    yf - ly / 2);
            label =
                    (int) dspFilter[chosenMode / 2][chosenDsp].getHighFreqCutoff() +
                    " Hz";
            g.drawString(label, (int) (r.getX() + r.getWidth() - dspSkirt),
                    yf - ly / 2);
        } else {
            // Label center frequency for CW and RTTY.
            label = (int) dspFilter[chosenMode / 2][chosenDsp].getCenterFreq() +
                    " Hz";
            bb = calculateStringBox(g, label);
            int cx = (int) (r.getX() + r.getWidth() / 2);
            g.drawString(label, cx - (int) (bb.getWidth() / 2), yf - ly / 2);
            g.drawLine(cx, yf, cx, yf + heightf);
        }
    }

    private void drawDspNotch(Graphics2D g2) {
        if (dspTrapezoids == null) {
            return;
        }
        Rectangle2D r = dspTrapezoids[chosenMode / 2][chosenDsp].getP().
                getBounds2D();
        int nx = (int) (r.getX() + r.getWidth() / 2);
        int ny = (int) r.getY();
        SlowShape tri = new SlowShape();
        tri.moveTo(0, 32);
        tri.lineTo(-8, 0);
        tri.lineTo(8, 0);
        tri.closePath();
        notchOn = dspFilter[chosenMode / 2][chosenDsp].isNotchFilter();
        tri.setFill(notchOn);
        tri.translate(nx, ny);
        g2.setStroke(new BasicStroke(1));
        tri.draw(g2);
    }

    private Rectangle2D calculateStringBox(Graphics g, String s) {
        FontMetrics fm = g.getFontMetrics();
        Rectangle2D r = fm.getStringBounds(s, g);
        return r;
    }

    private SlowShape[][] makeTrapezoids(FilterModel[][] filter,
            double baseAngle) {
        if (filter == null || filter[0][0] == null) {
            return null;
        }
        int f = filter[0].length; // # filters/mode
        int m = filter.length; // # modes
        SlowShape[][] s = new SlowShape[m][f];
        // Find widest filter.
        widestBw = filter[0][0].getBandwidth();
        for (int mInd = 0; mInd < m; mInd++) {
            for (int fInd = 0; fInd < f; fInd++) {
                if (filter[mInd][fInd].getBandwidth() > widestBw) {
                    widestBw = filter[mInd][fInd].getBandwidth();
                }
            }
        }
        // Create Trapezoid shapes.
        for (int mInd = 0; mInd < m; mInd++) {
            for (int fInd = 0; fInd < f; fInd++) {
                double ratio = filter[mInd][fInd].getBandwidth() / widestBw;
                int skirt = (int) (heightf / Math.tan(baseAngle));
                int width = (int) ((widthf - 2 * skirt) * ratio);
                s[mInd][fInd] = makeTrapezoid(width + 2 * skirt, heightf,
                        baseAngle);
                s[mInd][fInd].translate(xf, yf);
            }
        }
        return s;
    }

    private SlowShape makeTrapezoid(int width, int height, double baseAngle) {
        // Draw trapezoid outline.
        int skirt = (int) (height / Math.tan(baseAngle));
        SlowShape s = new SlowShape();
        s.setStroke(new BasicStroke(Integer.valueOf(2)));
        s.moveTo(0, height);
        s.lineTo(skirt, 0);
        s.lineTo(width - skirt, 0);
        s.lineTo(width, height);
        s.lineTo(0, height);
        return s;
    }

    private void updateComponentSizes() {
        int w = getWidth();
        int h = getHeight();
        // Center filter pic 10% from each corner.
        xf = (int) (0.1 * w);
        yf = (int) (0.2 * h);
        widthf = (int) (0.8 * w);
        heightf = (int) (0.6 * h);
        // Find width of base of skirt triangle.
        crystalSkirt = (int) (heightf / Math.tan(thetaCrystal));
        dspSkirt = (int) (heightf / Math.tan(thetaDsp));
        crystalTrapezoids = makeTrapezoids(crystalFilter, thetaCrystal);
        dspTrapezoids = makeTrapezoids(dspFilter, thetaDsp);
        if (dspTrapezoids != null && dspTrapezoids[0][0] != null) {
            for (int i = 0; i < dspTrapezoids[chosenMode / 2].length; i++) {
                updateFilterFreqs(i);
            }
        }
    }

    private int dspSkirtNarrow() {
        thetaDsp = Math.toRadians(80);
        dspSkirt = (int) (heightf / Math.tan(thetaDsp));
        return (int) (-2 * crystalSkirt + 2 * dspSkirt);
    }

    private int dspSkirtWide() {
        int delta = (int) (- 2 * dspSkirt + 2 * crystalSkirt);
        thetaDsp = thetaCrystal;
        dspSkirt = (int) (heightf / Math.tan(thetaDsp));
        return delta;
    }

    private int findSmallestTrapezoid(double x, double y) {
        int index = -1;
        double bw = Double.MAX_VALUE;
        for (int i = 0; i < crystalTrapezoids[chosenMode].length; i++) {
            double thisBw = crystalFilter[chosenMode][i].getBandwidth();
            if (crystalTrapezoids[chosenMode][i].contains(x, y) && thisBw < bw) {
                index = i;
                bw = thisBw;
            }
        }
        return index;
    }

    private void updateFilterFreqs(int i) {
        if (i == -1) {
            return;
        }
        double bw, hi, lo, ratio;
        // Update filter frequencies.
        SlowShape t = dspTrapezoids[chosenMode / 2][i];
        FilterModel f = dspFilter[chosenMode / 2][i];
        GeneralPath.Double path = t.getP();
        Rectangle2D r = path.getBounds2D();
        double w2 = 2 * crystalSkirt;
        // Set low frequency.
        ratio = ((r.getX() + dspSkirt) - (xf + crystalSkirt)) / (widthf - w2);
        lo = 500 + ratio * widestBw;
        dspFilter[chosenMode / 2][i].setLowFreqCutoff(lo);
        // Set high cutoff frequency.
        ratio = ((r.getX() - dspSkirt + r.getWidth()) - (xf + crystalSkirt)) /
                (widthf - w2);
        hi = 500 + ratio * widestBw;
        f.setHighFreqCutoff(hi);
//        dspFilter[chosenDsp].setHighFreqCutoff(lo + bw);
        // Set bandwidth.
        ratio = (r.getWidth() - 2 * dspSkirt) / (widthf - w2);
//        bw = widestBw * ratio;
//        dspFilter[chosenDsp].setBandwidth(bw);
        bw = hi - lo;
        f.setBandwidth(bw);
        // Set center frequency.
        f.setCenterFreq(lo + bw / 2);
    }

    public void setCwDenoiser(double gain) {
        for (int i = 0; i < 4; i++) {
            dspFilter[0][i].setDenoiserGain(gain);
        }
    }

    public void setSsbDenoiser(double gain) {
        for (int i = 0; i < 4; i++) {
            dspFilter[1][i].setDenoiserGain(gain);
        }
    }

    public void setNrBeta(int beta) {
        for (int m = 0; m < 2; m++) {
            for (int f = 0; f < 4; f++) {
                dspFilter[m][f].setNrBeta(beta);
            }
        }
    }

    public void setNrHold(int hold) {
        for (int m = 0; m < 2; m++) {
            for (int f = 0; f < 4; f++) {
                dspFilter[m][f].setNrHold(hold);
            }
        }
    }

    public void setNtBeta(int beta) {
        for (int f = 0; f < 4; f++) {
            dspFilter[1][f].setNtBeta(beta);
        }
    }

    public void setNtHold(int hold) {
        for (int f = 0; f < 4; f++) {
            dspFilter[1][f].setNtHold(hold);
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                mouseClickHandler(evt);
            }
            public void mousePressed(java.awt.event.MouseEvent evt) {
                mousePressedHandler(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                mouseReleasedHandler(evt);
            }
        });
        addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                resizeHandler(evt);
            }
        });
        addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
            public void mouseDragged(java.awt.event.MouseEvent evt) {
                mouseDragHandler(evt);
            }
            public void mouseMoved(java.awt.event.MouseEvent evt) {
                mouseMovedHandler(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 300, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents

    private void resizeHandler(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_resizeHandler
        updateComponentSizes();
    }//GEN-LAST:event_resizeHandler

    private void mouseClickHandler(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_mouseClickHandler
        int x = evt.getX();
        int y = evt.getY();
        if (!crystalMode) {
            if (dspTrapezoids[chosenMode / 2][chosenDsp].contains(x, y)) {
                GeneralPath.Double path =
                        dspTrapezoids[chosenMode / 2][chosenDsp].getP();
                Rectangle2D r = path.getBounds2D();
                int nx = (int) (r.getX() + r.getWidth() / 2);
                int ny = (int) r.getY();
                if (Math.abs(x - nx) < 20 && Math.abs(y - ny) < 20) {
                    notchOn = !notchOn;
                    dspFilter[chosenMode / 2][chosenDsp].setNotchFilter(notchOn);
                    if (main != null) {
                        main.updateMode();
                        main.enableAn(notchOn);
                    }
                    repaint();
                }
            }
        } else {
            chosenCrystal[chosenMode] = findSmallestTrapezoid(x, y);
            if (chosenCrystal[chosenMode] == -1) {
                main.enableDsps(false);
                main.setModeXtal();
            }
        }
    }//GEN-LAST:event_mouseClickHandler

    private void mouseDragHandler(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_mouseDragHandler
        boolean leftSkirt;
        int x = evt.getX();
        int y = evt.getY();
        if (crystalMode) {
            chosenCrystal[chosenMode] = findSmallestTrapezoid(x, y);
            if (chosenCrystal[chosenMode] == -1) {
                main.enableDsps(false);
            }
        } else {
            FilterModel d = dspFilter[chosenMode / 2][chosenDsp];
            SlowShape cf =
                    crystalTrapezoids[chosenMode][chosenCrystal[chosenMode]];
//            GeneralPath.Double cfp = cf.getP();
//            Rectangle2D cfpbb = cfp.getBounds2D();
            GeneralPath.Double path = dspTrapezoids[chosenMode / 2][chosenDsp].
                    getP();
            Rectangle2D r = path.getBounds2D();
//            double cLeft = cfpbb.getX();
//            double cRight = cfpbb.getX() + cfpbb.getWidth();
            double dLeft = r.getX();
            double dRight = r.getX() + r.getWidth();
            int deltaX = x - clickX;
            int w = (int) r.getWidth();
            leftSkirt = false;
            switch (editMode) {
                case '-':
                    return;
                case 'C': // CW left skirt adjust.
                    if (d.isCwNarrow() && clickX - x > 0) {
                        d.setCwNarrow(false);
                        int delta = crystalSkirt - dspSkirt;
                        int w1 = dspSkirtWide();
                        SlowShape s = makeTrapezoid(w + w1,
                                (int) r.getHeight(), thetaDsp);
                        dspTrapezoids[chosenMode / 2][chosenDsp] = s;
                        s.translate(r.getX() - delta, yf);
                    } else if (!d.isCwNarrow() && x - clickX > 0) {
                        d.setCwNarrow(true);
                        int w0 = dspSkirtNarrow();
                        SlowShape s = makeTrapezoid(w + w0,
                                (int) r.getHeight(), thetaDsp);
                        dspTrapezoids[chosenMode / 2][chosenDsp] = s;
                        s.translate(r.getX() + (crystalSkirt - dspSkirt), yf);
                    }
                    break;
                case 'c': // CW right skirt adjust.
                    if (d.isCwNarrow() && x - clickX > 0) {
                        d.setCwNarrow(false);
                        int delta = crystalSkirt - dspSkirt;
                        int w1 = dspSkirtWide();
                        SlowShape s = makeTrapezoid(w + w1,
                                (int) r.getHeight(), thetaDsp);
                        dspTrapezoids[chosenMode / 2][chosenDsp] = s;
                        s.translate(r.getX() - delta, yf);
                    } else if (!d.isCwNarrow() && clickX - x > 0) {
                        d.setCwNarrow(true);
                        int w0 = dspSkirtNarrow();
                        SlowShape s = makeTrapezoid(w + w0,
                                (int) r.getHeight(), thetaDsp);
                        dspTrapezoids[chosenMode / 2][chosenDsp] = s;
                        s.translate(r.getX() + (crystalSkirt - dspSkirt), yf);
                    }
                    updateFilterFreqs(chosenDsp);
                    break;
                case 'n':
                    int newNr = (int) (((double) heightf - (y - yf)) / nrStep);
                    int l = Math.min(4, newNr);
                    l = Math.max(0, l);
                    d.setNoiseReductionLevel(l);
                    nrDotCoords();
                    if (main != null) {
                        main.enableNr(l > 0);
                    }
                    break;
                case 'm':
//                    if (dLeft + deltaX < cLeft || dRight + deltaX > cRight) {
//                    if (dLeft + deltaX < 0 || dRight + deltaX > getWidth()) {
                    if (x < 0 || x > getWidth()) {
                        return;
                    }
                    AffineTransform transform = new AffineTransform();
                    transform.translate(deltaX, 0);
                    path.transform(transform);
                    updateFilterFreqs(chosenDsp);
                    break;
                case 'S':
                    deltaX = -deltaX;
                    leftSkirt = true;
                case 's':
                    int newWidth;
                    // Clicked in right skirt, so modify bandwidth.
                    if (chosenMode != LSB && chosenMode != USB) {
                        int newX = (int) dLeft - deltaX;
                        newWidth = (int) r.getWidth() + 2 * deltaX;
//                        if (newX >= cLeft && newWidth >= 2 * widthSkirt
//                                && newX + newWidth <= cRight) {
                        if (newX < 0 || newX + newWidth > getWidth()) {
                            return;
                        }
                        dspTrapezoids[chosenMode / 2][chosenDsp] =
                                makeTrapezoid(newWidth, heightf, thetaDsp);
                        dspTrapezoids[chosenMode / 2][chosenDsp].translate(newX,
                                yf);
                    } else {
//                        if ((leftSkirt && dLeft - deltaX < cLeft)
//                                || (!leftSkirt && dRight + deltaX > cRight)
//                                || (r.getWidth() + deltaX < 2 * widthSkirt)) {
//                            return;
//                        }
                        if ((leftSkirt && dLeft - deltaX < 0) || (!leftSkirt &&
                                dRight + deltaX > getWidth()) || (r.getWidth() +
                                deltaX < 2 * dspSkirt)) {
                            return;
                        }
                        newWidth = (int) r.getWidth() + deltaX;
                        dspTrapezoids[chosenMode / 2][chosenDsp] =
                                makeTrapezoid(newWidth, heightf, thetaDsp);
                        if (leftSkirt) {
                            dLeft -= deltaX;
                        }
                        dspTrapezoids[chosenMode / 2][chosenDsp].translate(dLeft,
                                yf);
                    }
                    updateFilterFreqs(chosenDsp);
                    break;
                default:
                    System.out.println(getClass() + ".mouseDragHandler: break!");
                    break;
            }
            clickX = x;
            clickY = y;
        }
        repaint();
    }//GEN-LAST:event_mouseDragHandler

    private void mousePressedHandler(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_mousePressedHandler
        int x = evt.getX();
        int y = evt.getY();
        if (crystalMode) {
            chosenCrystal[chosenMode] = findSmallestTrapezoid(x, y);
        } else {
            GeneralPath.Double path = dspTrapezoids[chosenMode / 2][chosenDsp].
                    getP();
            Rectangle2D r = path.getBounds2D();
            int dLeft = (int) r.getX();
            int dRight = (int) (dLeft + r.getWidth());
            int dEdge = (int) (r.getY() + r.getHeight());
            if ((chosenMode == CW || chosenMode == CW_REV) &&
                    Math.abs(x - dLeft) < 10 && Math.abs(y - dEdge) < 10) {
                editMode = 'C'; // CW left skirt adjust.
            } else if ((chosenMode == CW || chosenMode == CW_REV) && Math.
                    abs(x - dRight) < 10 && Math.abs(y - dEdge) < 10) {
                editMode = 'c'; // CW right skirt adjust.
            } else if ((chosenMode != RTTY) && (chosenMode != RTTY_REV) &&
                    Math.abs(x - nrX) < 10 && Math.abs(y - nrY) < 10) {
                editMode = 'n'; // Noise reduction level adjust.
            } else if (dspTrapezoids[chosenMode / 2][chosenDsp].contains(x,
                    y)) {
                int width = (int) r.getWidth() - 2 * dspSkirt;
                if (x <= r.getX() + dspSkirt) {
                    editMode = 'S'; // Skirt
                } else if (x >= r.getX() + width + dspSkirt) {
                    editMode = 's'; // Skirt
                } else {
                    editMode = 'm'; // Move
                }
                clickX = x;
                clickY = y;
            } else {
                editMode = '-'; // Not in edit mode.
                clickX = -1;
            }
        }
        repaint();
    }//GEN-LAST:event_mousePressedHandler

    private void mouseMovedHandler(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_mouseMovedHandler
        // TODO add your handling code here:
        int x = evt.getX();
        int y = evt.getY();
        if (!crystalMode) {
            GeneralPath.Double path = dspTrapezoids[chosenMode / 2][chosenDsp].
                    getP();
            Rectangle2D r = path.getBounds2D();
            int dLeft = (int) r.getX();
            int dRight = (int) (dLeft + r.getWidth());
            int dEdge = (int) (r.getY() + r.getHeight());
            if (chosenMode == CW || chosenMode == CW_REV) {
                if (Math.abs(x - dLeft) < 10 && Math.abs(y - dEdge) < 10) {
                    drawLeftDot = true;
                } else if (Math.abs(x - dRight) < 10 &&
                        Math.abs(y - dEdge) < 10) {
                    drawRightDot = true;
                } else {
                    drawLeftDot = drawRightDot = false;
                }
                repaint();
            }
            if (chosenMode != RTTY && chosenMode != RTTY_REV) {
                if (Math.abs(x - nrX) < 10 && Math.abs(y - nrY) < 10) {
                    drawNrDot = true;
                } else {
                    drawNrDot = false;
                }
                repaint();
            }
        }
    }//GEN-LAST:event_mouseMovedHandler

    private void mouseReleasedHandler(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_mouseReleasedHandler
        if (crystalMode) {
            main.setModeXtal();
        }
    }//GEN-LAST:event_mouseReleasedHandler
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
}
