/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package antbw;

import java.util.Observable;
import java.util.Observer;
import javax.swing.*;
import util.Serial;

/**
 *
 * @author mm
 */
public class AntBwPanel extends JPanel implements Observer {

    private Measurer measurer;
    private Serial rigPort, w1Port;
    private Thread measurerThread;

    /**
     * Creates new form AntBwPanel
     */
    public AntBwPanel() {
        initComponents();
        setEnabledMeasure(false);
        setEnabledRadio(false);
    }

    public void init() {

        rigPort = new Serial();
        w1Port = new Serial();
        measurer = new Measurer();
        measurer.setW1Port(w1Port);
        measurer.setRigPort(rigPort);
        measurer.addObserver(this);
        w1ComboBox.setEnabled(false);
        mkPortComboBoxes();
    }

    public void stopMeasurerThread() {
        if (measurerThread != null && measurerThread.isAlive()) {
            measurer.stop();
        }
    }

    private void setEnabledMeasure(boolean enable) {
        progressBar.setEnabled(enable);
        pauseButton.setEnabled(enable);
        playButton.setEnabled(enable);
        stateLabel.setEnabled(enable);
        stopButton.setEnabled(enable);
    }

    private void setEnabledRadio(boolean enabled) {
        startTextField.setEnabled(enabled);
        stopTextField.setEnabled(enabled);
        numSamplesTextField.setEnabled(enabled);
        txPowerTextField.setEnabled(enabled);
    }

    public void pause() {
        measurer.pause();
        playButton.setSelected(false);
        stopButton.setSelected(false);
        stateLabel.setText("Paused.");
    }

    public void play() {

        JFileChooser fc = new JFileChooser();
        int returnVal = fc.showOpenDialog(this);
        fc.setMultiSelectionEnabled(false);
        if (returnVal == JFileChooser.CANCEL_OPTION) {
            return;
        }
        measurer.setOutputFile(fc.getSelectedFile());
        measurer.play();
        pauseButton.setSelected(false);
        stopButton.setSelected(false);
        stateLabel.setText("Measuring...");
        measurerThread = new Thread(measurer);
        measurerThread.start();
    }

    public void stop() {
        measurer.stop();
        playButton.setSelected(false);
        pauseButton.setSelected(false);
        stateLabel.setText("Stopped.");
    }

    private void mkPortComboBoxes() {

        DefaultComboBoxModel m1 = new DefaultComboBoxModel();
        DefaultComboBoxModel m2 = new DefaultComboBoxModel();
        m1.addElement("Select serial port.");
        m2.addElement("Select serial port.");

        String[] ports = w1Port.getSerialPortNames();
        for (int i = 0; i < ports.length; i++) {
            m1.addElement(ports[i]);
            m2.addElement(ports[i]);
        }
        rigComboBox.setModel(m1);
        w1ComboBox.setModel(m2);
    }

    private void entryError(String msg) {
        JOptionPane.showMessageDialog(this, msg, "Entry error",
                JOptionPane.ERROR_MESSAGE);
    }

    private void checkNumbers(JTextField f) {
        double x;
        if (f.getText().isEmpty()) {
            return;
        }
        if (f == numSamplesTextField || f == txPowerTextField) {
            int n;
            try {
                n = Integer.valueOf(f.getText());
            } catch (NumberFormatException e) {
                f.selectAll();
                entryError("Integer needed.");
                return;
            }
            if (f == numSamplesTextField) {
                measurer.setNumSamples(n);
            } else if (f == txPowerTextField) {
                measurer.setTxPower(n);
            }
        } else {
            try {
                x = Double.valueOf(f.getText());
            } catch (NumberFormatException e) {
                f.selectAll();
                entryError("Real number expected.");
                return;
            }
            if (x < 0) {
                f.selectAll();
                entryError("Positive number needed.");
                return;
            }
            if (f == startTextField) {
                measurer.setFreqStart(x);
            } else if (f == stopTextField) {
                measurer.setFreqStop(x);
            }
        }
        if (numSamplesTextField.getText().isEmpty()
                || startTextField.getText().isEmpty()
                || stopTextField.getText().isEmpty()
                || txPowerTextField.getText().isEmpty()) {
            setEnabledMeasure(false);
            return;
        }
        setEnabledMeasure(true);
    }

    @Override
    public void update(Observable o, Object arg) {
        int n = (Integer) arg;
        if (n == 100) {
            stop();
            stateLabel.setText("Done.");
        }
        progressBar.setValue(n);
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        startTextField = new javax.swing.JTextField();
        jLabel1 = new javax.swing.JLabel();
        stopTextField = new javax.swing.JTextField();
        numSamplesTextField = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        txPowerTextField = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        jLabel8 = new javax.swing.JLabel();
        jLabel9 = new javax.swing.JLabel();
        rigComboBox = new javax.swing.JComboBox();
        w1ComboBox = new javax.swing.JComboBox();
        jPanel3 = new javax.swing.JPanel();
        stateLabel = new javax.swing.JLabel();
        progressBar = new javax.swing.JProgressBar();
        pauseButton = new javax.swing.JToggleButton();
        stopButton = new javax.swing.JToggleButton();
        playButton = new javax.swing.JToggleButton();
        closeButton = new javax.swing.JButton();

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Radio Set up"));

        startTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                textHandler(evt);
            }
        });
        startTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                focusGainedHandler(evt);
            }
            public void focusLost(java.awt.event.FocusEvent evt) {
                focusLostHandler(evt);
            }
        });

        jLabel1.setText("kHz");

        stopTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                textHandler(evt);
            }
        });
        stopTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                focusGainedHandler(evt);
            }
            public void focusLost(java.awt.event.FocusEvent evt) {
                focusLostHandler(evt);
            }
        });

        numSamplesTextField.setText("20");
        numSamplesTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                textHandler(evt);
            }
        });
        numSamplesTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                focusGainedHandler(evt);
            }
            public void focusLost(java.awt.event.FocusEvent evt) {
                focusLostHandler(evt);
            }
        });

        jLabel2.setText("kHz");

        jLabel4.setText("Start:");

        jLabel5.setText("Stop:");

        jLabel6.setText("No. Samples:");

        jLabel7.setText("Tx power:");

        txPowerTextField.setText("1");
        txPowerTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                textHandler(evt);
            }
        });
        txPowerTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                focusGainedHandler(evt);
            }
            public void focusLost(java.awt.event.FocusEvent evt) {
                focusLostHandler(evt);
            }
        });

        jLabel3.setText("W");

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel4)
                            .addComponent(jLabel5))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(stopTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 80, Short.MAX_VALUE)
                            .addComponent(startTextField))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel1)
                            .addComponent(jLabel2)))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel6)
                            .addComponent(jLabel7))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(numSamplesTextField)
                            .addComponent(txPowerTextField))
                        .addGap(4, 4, 4)
                        .addComponent(jLabel3)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(startTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel1)
                    .addComponent(jLabel4))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(stopTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel2)
                    .addComponent(jLabel5))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(numSamplesTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel6))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(txPowerTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel3)
                    .addComponent(jLabel7)))
        );

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder("Computer Set Up"));

        jLabel8.setText("K3 port:");

        jLabel9.setText("W1 port:");

        rigComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                comboHandler(evt);
            }
        });

        w1ComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                comboHandler(evt);
            }
        });

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel8)
                    .addComponent(jLabel9))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(rigComboBox, 0, 223, Short.MAX_VALUE)
                    .addComponent(w1ComboBox, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel8)
                    .addComponent(rigComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel9)
                    .addComponent(w1ComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder("Measure"));

        stateLabel.setText("Stopped.");

        pauseButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/antbw/pause.gif"))); // NOI18N
        pauseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                toggleButtonHandler(evt);
            }
        });

        stopButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/antbw/stop.gif"))); // NOI18N
        stopButton.setSelected(true);
        stopButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                toggleButtonHandler(evt);
            }
        });

        playButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/antbw/play.gif"))); // NOI18N
        playButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                toggleButtonHandler(evt);
            }
        });

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel3Layout.createSequentialGroup()
                        .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, 0, Short.MAX_VALUE)
                        .addContainerGap())
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel3Layout.createSequentialGroup()
                        .addGap(0, 0, Short.MAX_VALUE)
                        .addComponent(playButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(pauseButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(stopButton)
                        .addGap(6, 6, 6))
                    .addGroup(jPanel3Layout.createSequentialGroup()
                        .addComponent(stateLabel)
                        .addGap(0, 0, Short.MAX_VALUE))))
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(playButton)
                    .addComponent(pauseButton)
                    .addComponent(stopButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(stateLabel)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        closeButton.setText("Close");
        closeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonHandler(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 104, Short.MAX_VALUE)
                        .addComponent(closeButton))
                    .addComponent(jPanel2, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jPanel3, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jPanel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(closeButton))
                    .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void toggleButtonHandler(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_toggleButtonHandler
        JToggleButton b = (JToggleButton) evt.getSource();
        if (b == pauseButton) {
            pause();
        } else if (b == playButton) {
            // Prevent user from changing ports once measuring commences.
            rigComboBox.setEnabled(false);
            w1ComboBox.setEnabled(false);
            setEnabledRadio(false);
            play();
        } else if (b == stopButton) {
            rigComboBox.setEnabled(true);
            w1ComboBox.setEnabled(true);
            setEnabledRadio(true);
            progressBar.setValue(0);
            stop();
        }
    }//GEN-LAST:event_toggleButtonHandler

    private void comboHandler(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_comboHandler
        JComboBox cb = (JComboBox) evt.getSource();
        String selectedPort = (String) cb.getSelectedItem();
        if (cb == rigComboBox) {
            // Connect to rig.
            if (rigPort.setSerialPort("K3", selectedPort)) {
                w1ComboBox.setEnabled(true);
            }
        } else if (cb == w1ComboBox) {
            // Connect to watt meter.
            if (w1Port.setSerialPort("W1", selectedPort)) {
                setEnabledRadio(true);
            }
        }
    }//GEN-LAST:event_comboHandler

    private void textHandler(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_textHandler
        JTextField f = (JTextField) evt.getSource();
        checkNumbers(f);
    }//GEN-LAST:event_textHandler

    private void focusLostHandler(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_focusLostHandler
        JTextField f = (JTextField) evt.getSource();
        checkNumbers(f);
    }//GEN-LAST:event_focusLostHandler

    private void focusGainedHandler(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_focusGainedHandler
        JTextField f = (JTextField) evt.getSource();
        f.selectAll();
    }//GEN-LAST:event_focusGainedHandler

    private void buttonHandler(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonHandler
        JButton b = (JButton) evt.getSource();
        if (b == closeButton) {
            System.exit(0);
        }
    }//GEN-LAST:event_buttonHandler
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton closeButton;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JTextField numSamplesTextField;
    private javax.swing.JToggleButton pauseButton;
    private javax.swing.JToggleButton playButton;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JComboBox rigComboBox;
    private javax.swing.JTextField startTextField;
    private javax.swing.JLabel stateLabel;
    private javax.swing.JToggleButton stopButton;
    private javax.swing.JTextField stopTextField;
    private javax.swing.JTextField txPowerTextField;
    private javax.swing.JComboBox w1ComboBox;
    // End of variables declaration//GEN-END:variables
}
