package digital;
import java.awt.Rectangle;

/** Implement the power/peak button on the front of the virtual W1.  The
 * implementation toggles between average power and PEP, and the secondary
 * function toggles between regular and peak-hold modes.
 *
 * Mike Markowski<br>
 * March 2007
 */

public class ButtonPwr extends Button {

	/** Create a power/peak button specifying internally used button name (not
	 * displayed on-screen) and the geometry in pixels specifying where the
	 * button is on-screen. */
	ButtonPwr(String name, int x0, int y0, int xlen, int ylen) {
		this.name = name;
		geometry = new Rectangle(x0, y0, xlen, ylen);
		primary = 'A'; // Average power.
		secondary = 'N'; // Peak bargraph.
		pstr = "AVG";
		sstr = "";
	}

	/** For each tap, i.e., call to this method, the ButtonPwr toggles
	 * between average power and PEP.  The appropriate W1 single letter
	 * command is returned to put the W1 in the requested measurement state. */
    @Override
	public String getTapFn() {
		switch (primary) {
		case 'A':
			primary = 'P';
			pstr = "PEP";
			break;
		case 'P':
			primary = 'A';
			pstr = "AVG";
			break;
		}
		return "MN";
	}

	/** For each hold, i.e., call to this method, the ButtonPwr toggles
	 * between regular and peak-hold display of measurements.  The
	 * appropriate W1 single letter command is returned to put the W1 in the
	 * requested measurement state. */
    @Override
	public String getHoldFn() {
		switch (secondary) {
		case 'N':
		case 'R':
			secondary = 'O';
			sstr = "HOLD";
			break;
		case 'O':
		case 'P':
			secondary = 'N';
			sstr = "";
			break;
		}
		return "P";
	}

}
