/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package tiny;

import java.awt.Color;
import java.awt.Graphics;
import javax.swing.JLabel;
import w1.PowerMeter;

/**
 *
 * @author mm
 */
public class MiniMeter extends javax.swing.JPanel implements PowerMeter {

    private boolean units_W; // True => Watts, false => dBm.
    private double fwd_W, rfl_W;

    /**
     * Creates new form MiniMeter
     */
    public MiniMeter() {
        initComponents();
        swrPanel.setBackground(Color.GREEN);
        units_W = true;
    }

    @Override
    public void setFwd_W(double fwd_W) {
        this.fwd_W = fwd_W;
    }

    @Override
    public void setRfl_W(double rfl_W) {
        this.rfl_W = rfl_W;
        repaint();
    }

    @Override
    public boolean isUnits_W() {
        return units_W;
    }

    @Override
    public void setUnits_W(boolean units_W) {
        this.units_W = units_W;
    }

    @Override
    public void paintComponent(Graphics g) {

        double swr = calcSwr(fwd_W, rfl_W);

        String fwdStr, rflStr;

        if (!units_W) {
            fwdStr = fwd_W == 0 ? "-Inf" : formatPower(W_to_dBm(fwd_W));
            rflStr = rfl_W == 0 ? "-Inf" : formatPower(W_to_dBm(rfl_W));
            fwdUnitsLabel.setText("dBm");
            rflUnitsLabel.setText("dBm");
        } else {
            fwdStr = formatPower(fwd_W);
            rflStr = formatPower(rfl_W);
            fwdUnitsLabel.setText("W");
            rflUnitsLabel.setText("W");
        }
        Color alertLevel;
        if (swr <= 1.5) {
            alertLevel = Color.GREEN;
        } else if (swr <= 2.5) {
            alertLevel = Color.YELLOW;
        } else {
            alertLevel = Color.RED;
        }
        swrPanel.setBackground(alertLevel);
        fwdLabel.setText(fwdStr);
        rflLabel.setText(rflStr);
        swrLabel.setText(String.format("%.1f", calcSwr(fwd_W, rfl_W)));
    }

    private double W_to_dBm(double watts) {
        double mW = watts * 1e3;
        return 10 * Math.log10(mW);
    }

    private double calcSwr(double fwd_W, double rfl_W) {

        if (rfl_W == 0) {
            return 1;
        }
        double g = Math.sqrt(rfl_W / fwd_W);
        return (1 + g) / (1 - g);
    }

    private String formatPower(double power) {

        if (power < 10) {
            return String.format("%.2f", power);
        } else if (power < 100) {
            return String.format("%.1f", power);
        } else {
            return String.format("%d", Integer.valueOf((int) (0.5 + power)));
        }
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        fwdLabel = new javax.swing.JLabel();
        fwdUnitsLabel = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        rflLabel = new javax.swing.JLabel();
        rflUnitsLabel = new javax.swing.JLabel();
        swrPanel = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        swrLabel = new javax.swing.JLabel();

        setMaximumSize(new java.awt.Dimension(211, 60));
        setMinimumSize(new java.awt.Dimension(211, 60));

        jPanel1.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        jPanel1.setMaximumSize(new java.awt.Dimension(134, 60));
        jPanel1.setMinimumSize(new java.awt.Dimension(134, 60));
        jPanel1.setPreferredSize(new java.awt.Dimension(134, 60));

        jLabel1.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N
        jLabel1.setText("Fwd:");

        fwdLabel.setFont(new java.awt.Font("Dialog", 1, 14)); // NOI18N
        fwdLabel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        fwdLabel.setText("44.4");

        fwdUnitsLabel.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N
        fwdUnitsLabel.setText("dBm");

        jLabel4.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N
        jLabel4.setText("Refl:");

        rflLabel.setFont(new java.awt.Font("Dialog", 1, 14)); // NOI18N
        rflLabel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        rflLabel.setText("4.4");

        rflUnitsLabel.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N
        rflUnitsLabel.setText("dBm");

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                        .addComponent(jLabel1)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 12, Short.MAX_VALUE)
                        .addComponent(fwdLabel))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                        .addComponent(jLabel4)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(rflLabel)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(rflUnitsLabel, javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(fwdUnitsLabel, javax.swing.GroupLayout.Alignment.TRAILING))
                .addContainerGap(12, Short.MAX_VALUE))
        );

        jPanel1Layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {fwdLabel, rflLabel});

        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGap(6, 6, 6)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(fwdLabel)
                    .addComponent(fwdUnitsLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel4)
                    .addComponent(rflUnitsLabel)
                    .addComponent(rflLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
        );

        swrPanel.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        swrPanel.setMaximumSize(new java.awt.Dimension(89, 60));
        swrPanel.setPreferredSize(new java.awt.Dimension(89, 60));

        jLabel2.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N
        jLabel2.setText("SWR:");

        swrLabel.setFont(new java.awt.Font("Dialog", 1, 14)); // NOI18N
        swrLabel.setText("9.9");

        javax.swing.GroupLayout swrPanelLayout = new javax.swing.GroupLayout(swrPanel);
        swrPanel.setLayout(swrPanelLayout);
        swrPanelLayout.setHorizontalGroup(
            swrPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(swrPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(swrLabel)
                .addContainerGap(19, Short.MAX_VALUE))
        );
        swrPanelLayout.setVerticalGroup(
            swrPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(swrPanelLayout.createSequentialGroup()
                .addGap(20, 20, 20)
                .addGroup(swrPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2)
                    .addComponent(swrLabel))
                .addContainerGap(21, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(0, 0, 0)
                .addComponent(swrPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(swrPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel fwdLabel;
    private javax.swing.JLabel fwdUnitsLabel;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JLabel rflLabel;
    private javax.swing.JLabel rflUnitsLabel;
    private javax.swing.JLabel swrLabel;
    private javax.swing.JPanel swrPanel;
    // End of variables declaration//GEN-END:variables

    @Override
    public void setDecayRate(char decaySpeed) {
    }

    @Override
    public void setFirmwareVersion(String fw) {
    }

    @Override
    public boolean isPeakHold() {
        return false;
    }

    @Override
    public void setPeakHold(boolean hold) {
    }

    @Override
    public boolean isMonochrome() {
        return false;
    }

    @Override
    public void setMonochrome(boolean monochrome) {
    }

    @Override
    public boolean isLedsOn() {
        return false;
    }

    @Override
    public void setLedsOn(boolean ledsOn) {
    }

    @Override
    public boolean isPEP() {
        return false;
    }

    @Override
    public void setPEP(boolean pep) {
    }

    @Override
    public void startFrameTimer() {
    }

    @Override
    public void stopFrameTimer() {
    }

    @Override
    public void setDropRate(char dropRate) {
    }

    @Override
    public void setRange(char range) {
    }
}
